#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
频谱图分析模块

提供音频文件频谱图分析和显示功能
"""

import numpy as np
import matplotlib.pyplot as plt
from scipy.io import wavfile
from scipy import signal
import soundfile as sf
from typing import Dict, Any, Tuple, Optional
from matplotlib import font_manager as fm

# 添加Times New Roman字体设置
def set_plot_style():
    """
    设置绘图样式，使用Times New Roman字体
    """
    # 设置全局字体为Times New Roman
    plt.rcParams['font.family'] = 'Times New Roman'
    plt.rcParams['mathtext.fontset'] = 'stix'  # 数学公式字体
    plt.rcParams['axes.unicode_minus'] = False  # 正确显示负号


def analyze_spectrogram(audio_file: str, 
                        channel: int = 0, 
                        window_size: Optional[int] = None,
                        overlap: float = 0.5) -> Dict[str, Any]:
    """
    分析音频文件的频谱图

    参数:
        audio_file: 音频文件路径
        channel: 要分析的通道 (0=左, 1=右)
        window_size: 窗口大小 (None 表示使用默认值)
        overlap: 窗口重叠比例 (0.0-1.0)

    返回:
        包含频谱图数据的字典
    """
    # 读取音频文件
    try:
        sample_rate, data = wavfile.read(audio_file)
        # 检查文件类型，如果不是WAV，使用soundfile
        if data.dtype == 'float32' or data.dtype == 'float64':
            data = np.iinfo(np.int16).max * data
    except:
        # 如果wavfile无法读取，尝试使用soundfile
        data, sample_rate = sf.read(audio_file)
        # 将数据归一化到16位整数范围
        if data.dtype == 'float32' or data.dtype == 'float64':
            data = np.iinfo(np.int16).max * data

    # 如果是立体声，取指定的通道
    if data.ndim > 1:
        if channel >= data.shape[1]:
            channel = 0  # 如果指定的通道不可用，默认使用第一个
        data = data[:, channel]

    # 使用默认窗口参数
    if window_size is None:
        window_size = 256  # 使用scipy.signal.spectrogram的默认值

    # 计算频谱
    frequencies, times, Sxx = signal.spectrogram(
        data, 
        fs=sample_rate,
        nperseg=window_size,
        noverlap=int(window_size * overlap)
    )

    return {
        'audio_file': audio_file,
        'sample_rate': sample_rate,
        'data': data,
        'frequencies': frequencies,
        'times': times,
        'spectrogram': Sxx,
        'channel': channel,
        'window_size': window_size,
        'overlap': overlap
    }


def plot_spectrogram(spec_data: Dict[str, Any], 
                     use_log_scale: bool = True,
                     freq_limit: Optional[int] = None,
                     figsize: Tuple[int, int] = (12, 4),
                     save_path: Optional[str] = None,
                     vmin: float = -100,  # 添加最小值参数
                     vmax: float = -50    # 添加最大值参数
                     ) -> plt.Figure:
    """
    Plot spectrogram of audio data

    Args:
        spec_data: Spectrogram data from analyze_spectrogram
        use_log_scale: Whether to use log scale display
        freq_limit: Frequency limit (Hz), None for full range
        figsize: Figure size
        save_path: If provided, save figure to this path
        vmin: Minimum value for color scale (dB)
        vmax: Maximum value for color scale (dB)

    Returns:
        matplotlib figure object
    """
    frequencies = spec_data['frequencies']
    times = spec_data['times']
    Sxx = spec_data['spectrogram']
    
    # 设置绘图样式
    set_plot_style()
    
    # 创建图形
    fig = plt.figure(figsize=figsize)
    
    # 限制频率范围
    if freq_limit is not None and freq_limit > 0:
        freq_mask = frequencies <= freq_limit
        frequencies = frequencies[freq_mask]
        Sxx = Sxx[freq_mask, :]
    
    # 计算显示数据
    if use_log_scale:
        # 加入1e-10防止log(0)
        display_data = 10 * np.log10(Sxx + 1e-10)
        colorbar_label = 'Intensity [dB]'  # 改为 Intensity
    else:
        display_data = Sxx
        colorbar_label = 'Intensity'
    
    # 绘制频谱图，添加颜色范围控制
    plt.pcolormesh(times, frequencies, display_data, 
                  shading='gouraud',
                  vmin=vmin,
                  vmax=vmax)
    plt.colorbar(label=colorbar_label)
    
    # 设置标题和标签
    audio_file = spec_data['audio_file'].split("/")[-1]
    plt.title(f"Spectrogram: {audio_file}", pad=10)
    plt.xlabel("Time [s]")
    plt.ylabel("Frequency [Hz]")
    
    # 优化布局
    plt.tight_layout()
    
    # 保存图像
    if save_path:
        plt.savefig(save_path, dpi=300, bbox_inches='tight')
    
    return fig


def print_spectrogram_info(spec_data: Dict[str, Any]) -> None:
    """
    打印频谱图分析信息

    参数:
        spec_data: 从analyze_spectrogram获取的频谱图数据
    """
    print(f"Audio file: {spec_data['audio_file']}")
    print(f"Sample rate: {spec_data['sample_rate']} Hz")
    print(f"Data length: {len(spec_data['data'])} samples")
    print(f"Duration: {len(spec_data['data']) / spec_data['sample_rate']:.2f} seconds")
    print(f"Frequency resolution: {len(spec_data['frequencies'])} points")
    print(f"Time resolution: {len(spec_data['times'])} frames")
    print(f"Window size: {spec_data['window_size']} samples")
    print(f"Overlap ratio: {spec_data['overlap']:.2f}")


def display_spectrogram(audio_file: str, 
                        channel: int = 0,
                        window_size: Optional[int] = None,
                        overlap: float = 0.75,
                        use_log_scale: bool = True,
                        freq_limit: Optional[int] = None,
                        figsize: Tuple[int, int] = (12, 4),
                        save_path: Optional[str] = None) -> None:
    """
    Analyze and display spectrogram of audio file
    
    Args:
        audio_file: Path to audio file
        channel: Channel to analyze (0=left, 1=right)
        window_size: Window size (None for auto)
        overlap: Window overlap ratio (0.0-1.0)
        use_log_scale: Whether to use log scale display
        freq_limit: Frequency limit (Hz), None for full range
        figsize: Figure size
        save_path: If provided, save figure to this path
    """
    # 分析频谱图
    spec_data = analyze_spectrogram(
        audio_file=audio_file,
        channel=channel,
        window_size=window_size,
        overlap=overlap
    )
    
    # 打印信息
    print_spectrogram_info(spec_data)
    
    # 绘制并显示频谱图
    fig = plot_spectrogram(
        spec_data=spec_data,
        use_log_scale=use_log_scale,
        freq_limit=freq_limit,
        figsize=figsize,
        save_path=save_path
    )
    
    plt.show()
    
    return spec_data 


#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
WavX 命令行界面

提供命令行访问WavX库功能的入口点
"""

import argparse
import sys
from typing import List, Optional

from . import __version__
from .analysis import amplitude
from .analysis import spectrogram
from .processing import normalization
from .utils.logo import print_logo


def main(args: Optional[List[str]] = None) -> int:
    """
    命令行入口函数
    
    参数:
        args: 命令行参数列表。如果为None，则使用sys.argv
        
    返回:
        int: 退出码。0表示成功，非零表示错误
    """
    parser = argparse.ArgumentParser(
        description=f"WavX v{__version__} - A Simple Acoustic Agile Processing Library",
        formatter_class=argparse.RawDescriptionHelpFormatter
    )
    
    parser.add_argument(
        '--version', action='version',
        version=f'WavX {__version__}'
    )
    
    # 创建子命令
    subparsers = parser.add_subparsers(dest='command', help='可用命令')
    
    # 振幅分析命令
    amplitude_parser = subparsers.add_parser('amplitude', help='分析音频文件的振幅信息')
    amplitude_parser.add_argument('audio_file', help='要分析的音频文件路径')
    amplitude_parser.add_argument('--window', type=float, default=50.0, help='RMS窗口大小(毫秒)')
    amplitude_parser.add_argument('--no-dc', action='store_false', dest='consider_dc', help='不考虑DC偏移')
    
    # 频谱图分析命令
    spectrogram_parser = subparsers.add_parser('spectrogram', help='分析并显示音频文件的频谱图')
    spectrogram_parser.add_argument('audio_file', help='要分析的音频文件路径')
    spectrogram_parser.add_argument('--channel', '-c', type=int, default=0, help='要分析的通道 (0=左, 1=右)')
    spectrogram_parser.add_argument('--window', '-w', type=int, help='窗口大小 (默认自动设置)')
    spectrogram_parser.add_argument('--overlap', '-o', type=float, default=0.75, help='窗口重叠比例 (0.0-1.0)')
    spectrogram_parser.add_argument('--linear', action='store_false', dest='log_scale', help='使用线性刻度而不是对数刻度')
    spectrogram_parser.add_argument('--freq-limit', '-f', type=int, help='频率上限 (Hz)')
    spectrogram_parser.add_argument('--save', '-s', help='保存频谱图的文件路径')
    
    # RMS标准化命令
    normalize_parser = subparsers.add_parser('normalize', help='将音频文件标准化到指定的RMS电平')
    normalize_parser.add_argument('input_file', help='输入音频文件路径')
    normalize_parser.add_argument('output_file', help='输出音频文件路径')
    normalize_parser.add_argument('--target', '-t', type=float, default=-20.0, help='目标RMS电平(dB FS)，默认为-20dB')
    normalize_parser.add_argument('--reference', '-r', choices=['square', 'sine'], default='square', 
                                 help='参考信号类型，可选"square"或"sine"，默认为"square"')
    normalize_parser.add_argument('--freq', '-f', type=float, default=1000.0, help='参考信号频率(Hz)，默认为1000Hz')
    normalize_parser.add_argument('--duration', '-d', type=float, default=1.0, help='参考信号持续时间(秒)，默认为1秒')
    
    # 解析参数
    parsed_args = parser.parse_args(args)
    
    # 如果没有提供命令，显示LOGO和帮助信息
    if not parsed_args.command:
        print_logo()
        parser.print_help()
        return 0
    
    # 处理命令
    if parsed_args.command == 'amplitude':
        try:
            # 分析音频文件
            print(f"分析音频文件: {parsed_args.audio_file}")
            print(f"RMS窗口大小: {parsed_args.window} ms")
            print(f"考虑DC偏移: {parsed_args.consider_dc}")
            print("正在分析...")
            
            # 调用分析函数
            info = amplitude.analyze_amplitude(
                parsed_args.audio_file,
                rms_window_ms=parsed_args.window,
                consider_dc=parsed_args.consider_dc
            )
            
            # 打印结果
            amplitude.print_amplitude_info(info)
            
        except Exception as e:
            print(f"错误: {e}", file=sys.stderr)
            return 1
    
    elif parsed_args.command == 'spectrogram':
        try:
            # 分析并显示频谱图
            print(f"分析音频文件: {parsed_args.audio_file}")
            print(f"通道: {parsed_args.channel}")
            print(f"使用对数刻度: {parsed_args.log_scale}")
            if parsed_args.freq_limit:
                print(f"频率上限: {parsed_args.freq_limit} Hz")
            if parsed_args.save:
                print(f"保存到: {parsed_args.save}")
            print("正在生成频谱图...")
            
            # 调用频谱图函数
            spectrogram.display_spectrogram(
                audio_file=parsed_args.audio_file,
                channel=parsed_args.channel,
                window_size=parsed_args.window,
                overlap=parsed_args.overlap,
                use_log_scale=parsed_args.log_scale,
                freq_limit=parsed_args.freq_limit,
                save_path=parsed_args.save
            )
            
        except Exception as e:
            print(f"错误: {e}", file=sys.stderr)
            return 1
    
    elif parsed_args.command == 'normalize':
        try:
            # 标准化音频文件
            print(f"正在将音频文件 '{parsed_args.input_file}' 标准化到 {parsed_args.target} dB FS...")
            print(f"使用参考信号: {parsed_args.reference}波 {parsed_args.freq} Hz")
            print(f"输出文件: {parsed_args.output_file}")
            
            # 调用标准化函数
            info = normalization.normalize_to_target(
                input_file=parsed_args.input_file,
                output_file=parsed_args.output_file,
                target_rms_db=parsed_args.target,
                reference_type=parsed_args.reference,
                reference_duration=parsed_args.duration,
                reference_freq=parsed_args.freq
            )
            
            # 打印结果
            normalization.print_normalization_info(info)
            
        except Exception as e:
            print(f"错误: {e}", file=sys.stderr)
            return 1
    
    return 0


if __name__ == "__main__":
    sys.exit(main())


#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
WavX - 音频分析和处理工具库

WavX是一个模块化的音频处理库，提供各种声学分析和处理功能。
"""

__version__ = '0.1.6'
__author__ = 'Chord'

# 导入子模块
from . import analysis
from . import utils
from . import processing

# 设置日志
import logging
logging.getLogger(__name__).addHandler(logging.NullHandler())

# 如果检测到是通过pip安装的，则在安装完成后显示LOGO
import os
import sys

def _is_pip_install():
    """检查是否是通过pip install命令安装"""
    return any(arg.startswith('install') for arg in sys.argv)

# 如果是通过pip install安装并且不是在setup.py的上下文中
if _is_pip_install() and 'setup.py' not in sys.argv[0].lower():
    try:
        # 尝试调用post_install函数
        import subprocess
        import atexit
        
        # 注册退出时调用wavx-welcome
        def _show_logo_on_exit():
            try:
                subprocess.call([sys.executable, '-m', 'scripts'])
            except:
                pass
        
        atexit.register(_show_logo_on_exit)
    except:
        pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
安装脚本工具

用于pip安装过程中的自定义处理
"""

def post_install():
    """
    安装后处理，显示欢迎信息和LOGO
    """
    try:
        from wavx.utils.logo import print_install_message
        print_install_message()
    except ImportError:
        # 如果无法导入WavX，则使用一个简单的消息
        print("文档: https://github.com/JiangYain/WavX")
        print("使用 'wavx --help' 查看命令行工具的使用方法")

if __name__ == "__main__":
    post_install() 


#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Waveform Analysis Module

Provides functionality for audio waveform visualization
"""

import numpy as np
import matplotlib.pyplot as plt
from scipy.io import wavfile
import soundfile as sf
from typing import Dict, Any, Tuple, Optional

def set_plot_style():
    """
    Set plot style using Times New Roman font
    """
    plt.rcParams['font.family'] = 'Times New Roman'
    plt.rcParams['mathtext.fontset'] = 'stix'  # Math formula font
    plt.rcParams['axes.unicode_minus'] = False  # Correct minus sign display

def analyze_waveform(audio_file: str, channel: int = 0) -> Dict[str, Any]:
    """
    Analyze audio file waveform

    Args:
        audio_file: Path to audio file
        channel: Channel to analyze (0=left, 1=right)

    Returns:
        Dictionary containing waveform data
    """
    # Read audio file
    try:
        sample_rate, data = wavfile.read(audio_file)
        # Check file type, use soundfile if not WAV
        if data.dtype == 'float32' or data.dtype == 'float64':
            data = np.iinfo(np.int16).max * data
    except:
        # Try soundfile if wavfile fails
        data, sample_rate = sf.read(audio_file)
        # Normalize to 16-bit integer range
        if data.dtype == 'float32' or data.dtype == 'float64':
            data = np.iinfo(np.int16).max * data

    # If stereo, take specified channel
    if data.ndim > 1:
        if channel >= data.shape[1]:
            channel = 0  # Use first channel if specified is unavailable
        data = data[:, channel]

    # Generate time axis
    time = np.arange(len(data)) / sample_rate

    return {
        'audio_file': audio_file,
        'sample_rate': sample_rate,
        'data': data,
        'time': time,
        'channel': channel
    }

def plot_waveform(waveform_data: Dict[str, Any],
                  figsize: Tuple[int, int] = (12, 4),
                  save_path: Optional[str] = None) -> plt.Figure:
    """
    Plot audio waveform

    Args:
        waveform_data: Waveform data from analyze_waveform
        figsize: Figure size
        save_path: If provided, save figure to this path

    Returns:
        matplotlib figure object
    """
    # Set plot style
    set_plot_style()

    # Create figure
    fig = plt.figure(figsize=figsize)

    # Plot waveform
    plt.plot(waveform_data['time'], waveform_data['data'])

    # Set title and labels
    audio_file = waveform_data['audio_file'].split("/")[-1]
    plt.title(f"Waveform: {audio_file}", pad=10)
    plt.xlabel("Time [s]")
    plt.ylabel("Amplitude")
    plt.grid(True)

    # Optimize layout
    plt.tight_layout()

    # Save figure
    if save_path:
        plt.savefig(save_path, dpi=300, bbox_inches='tight')

    return fig

def print_waveform_info(waveform_data: Dict[str, Any]) -> None:
    """
    Print waveform analysis information

    Args:
        waveform_data: Waveform data from analyze_waveform
    """
    print(f"Audio file: {waveform_data['audio_file']}")
    print(f"Sample rate: {waveform_data['sample_rate']} Hz")
    print(f"Data length: {len(waveform_data['data'])} samples")
    print(f"Duration: {len(waveform_data['data']) / waveform_data['sample_rate']:.2f} seconds")
    print(f"Channel: {waveform_data['channel']}")

def display_waveform(audio_file: str,
                    channel: int = 0,
                    figsize: Tuple[int, int] = (12, 4),
                    save_path: Optional[str] = None) -> None:
    """
    Analyze and display audio file waveform

    Args:
        audio_file: Path to audio file
        channel: Channel to analyze (0=left, 1=right)
        figsize: Figure size
        save_path: If provided, save figure to this path
    """
    # Analyze waveform
    waveform_data = analyze_waveform(
        audio_file=audio_file,
        channel=channel
    )

    # Print information
    print_waveform_info(waveform_data)

    # Plot and display waveform
    fig = plot_waveform(
        waveform_data=waveform_data,
        figsize=figsize,
        save_path=save_path
    )

    plt.show()

    return waveform_data 


#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
WavX 音频分析模块

包含各种音频分析功能的模块集合
"""

from . import amplitude
from . import spectrogram
from . import waveform

# Export commonly used functions for convenience
from .amplitude import analyze_amplitude, print_amplitude_info
from .spectrogram import analyze_spectrogram, display_spectrogram
from .waveform import analyze_waveform, display_waveform



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
WavX Spectrogram Example

Demonstrates how to generate and display spectrograms using WavX
"""

import os
import sys
import argparse
import matplotlib.pyplot as plt

# Add project root to Python path to import wavx package
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

import wavx


def main():
    # Parse command line arguments
    parser = argparse.ArgumentParser(description='WavX Spectrogram Example')
    parser.add_argument('audio_file', help='Path to audio file')
    parser.add_argument('--channel', '-c', type=int, default=0, help='Analysis channel (0=left, 1=right)')
    parser.add_argument('--freq-limit', '-f', type=int, default=8000, help='Frequency limit (Hz)')
    parser.add_argument('--linear', action='store_true', help='Use linear scale instead of log scale')
    parser.add_argument('--save', '-s', help='Path to save spectrogram (e.g., spectrogram.png)')
    args = parser.parse_args()

    # Basic method: Direct analysis and display
    print("Method 1: Using integrated function to display spectrogram")
    wavx.analysis.spectrogram.display_spectrogram(
        audio_file=args.audio_file,
        channel=args.channel,
        use_log_scale=not args.linear,
        freq_limit=args.freq_limit,
        save_path=args.save
    )

    # Advanced method: Step by step processing
    print("\nMethod 2: Step by step spectrogram processing")
    # 1. Analyze spectrogram
    spec_data = wavx.analysis.spectrogram.analyze_spectrogram(
        audio_file=args.audio_file,
        channel=args.channel
    )
    
    # 2. Print spectrogram information
    wavx.analysis.spectrogram.print_spectrogram_info(spec_data)
    
    # 3. Plot spectrogram
    fig = wavx.analysis.spectrogram.plot_spectrogram(
        spec_data=spec_data,
        use_log_scale=not args.linear,
        freq_limit=args.freq_limit,
        figsize=(10, 6),
        save_path=None  # Don't save, just display
    )
    
    # 4. Add custom plot elements
    plt.axhline(y=1000, color='r', linestyle='--', label='1kHz Reference')
    plt.legend()
    
    # 5. Display modified plot
    plt.tight_layout()
    plt.show()


if __name__ == "__main__":
    main() 


#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
WavX LOGO模块

提供显示WavX LOGO的功能
"""

def print_logo(with_version=True):
    """
    打印WavX的ASCII艺术LOGO
    
    参数:
        with_version: 是否显示版本信息
    """
    from .. import __version__
    
    logo = """
 ____      ____  _  ____   ____   ____  ____  
|_  _|    |_  _|/ \|_  _| |_  _| |_  _||_  _| 
  \ \  /\  / / / _ \ \ \   / /     \ \  / /   
   \ \/  \/ / / ___ \ \ \ / /       > `' <    
    \  /\  /_/ /   \ \_\ ' /      _/ /'`\ \_  
     \/  \/|____| |____|\_/      |____||____| 
     
    """
    
    if with_version:
        from .. import __version__
        logo += f"\n      v{__version__}\n"
    
    print(logo)

def print_install_message():
    """
    打印安装完成后的欢迎消息和LOGO
    """
    print_logo()
    print("文档: https://github.com/chord-chord/wavx")
    print("使用 'wavx --help' 查看命令行工具的使用方法")
